# The command used to play streams
# You can add arguments to the command a la Python's subprocess module
PLAYER = ['/usr/bin/mpv']

# Respect default terminal colors in curses interface.
DEFAULT_TERMINAL_COLORS = False
